@@ -61,7 +61,7 @@ PopupBaseMenuItem.prototype = {
     },
 
     _onStyleChanged: function (actor) {
-        this._spacing = actor.get_theme_node().get_length('spacing');
+        this._spacing = Math.round(actor.get_theme_node().get_length('spacing'));
     },
 
     _onButtonReleaseEvent: function (actor, event) {
@@ -765,6 +765,7 @@ PopupMenuBase.prototype = {
         } else {
             this.box = new St.BoxLayout({ vertical: true });
         }
+        this.box.connect_after('queue-relayout', Lang.bind(this, this._menuQueueRelayout));
 
         this.isOpen = false;
 
@@ -893,6 +894,16 @@ PopupMenuBase.prototype = {
         }
     },
 
+    // Because of the above column-width funniness, we need to do a
+    // queue-relayout on every item whenever the menu itself changes
+    // size, to force clutter to drop its cached size requests. (The
+    // menuitems will in turn call queue_relayout on their parent, the
+    // menu, but that call will be a no-op since the menu already
+    // has a relayout queued, so we won't get stuck in a loop.
+    _menuQueueRelayout: function() {
+        this.box.get_children().map(function (actor) { actor.queue_relayout(); });
+    },
+
     addActor: function(actor) {
         this.box.add(actor);
     },
